package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmto;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmtoKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerie;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePf;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerieKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePfKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmto_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DpianificFerieAmmtoCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DpianificFerieAmmtoRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmto_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmtoKey_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerieKey_;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePfKey_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePfKey_;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePfKey_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePfKey_;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianiFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerieKey_;

import org.springframework.data.domain.PageImpl;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmtoKey_;

import java.util.stream.Collectors;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichPianifFerieAmmto;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DperiodiAmmtoPf;
import it.micegroup.voila2runtime.entity.GenericEntity;
import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DpianificFerieAmmtoServiceImpl extends BaseServiceImpl implements DpianificFerieAmmtoService {

	private final DpianificFerieAmmtoRepository dpianificFerieAmmtoRepository;
	private final FilterService<DpianificFerieAmmto> filterService;

	// CHILD SERVICES
	private final DrichPianifFerieAmmtoService drichPianifFerieAmmtoService;
	private final DperiodiAmmtoPfService dperiodiAmmtoPfService;

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DpianificFerieAmmtoServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DpianificFerieAmmto> findAll(Pageable pageable) {
		return dpianificFerieAmmtoRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DpianificFerieAmmto> findAll() {
		return dpianificFerieAmmtoRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DpianificFerieAmmto> findByCriteria(DpianificFerieAmmtoCriteria criteria, Pageable pageable) {
		final Specification<DpianificFerieAmmto> specification = createSpecification(criteria);
		return dpianificFerieAmmtoRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DpianificFerieAmmto> findByCriteria(DpianificFerieAmmtoCriteria criteria) {
		final Specification<DpianificFerieAmmto> specification = createSpecification(criteria);
		return dpianificFerieAmmtoRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DpianificFerieAmmto> findById(DpianificFerieAmmtoKey id) {
		return dpianificFerieAmmtoRepository.findByDpianificFerieAmmtoKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DpianificFerieAmmtoKey id) {
		return dpianificFerieAmmtoRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DpianificFerieAmmto insert(@Valid DpianificFerieAmmto entity) {
		return dpianificFerieAmmtoRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DpianificFerieAmmto update(@Valid DpianificFerieAmmto entity) {
		return dpianificFerieAmmtoRepository.save(entity);
	}

	@Override
	public void delete(DpianificFerieAmmto entity) {
		dpianificFerieAmmtoRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DpianificFerieAmmtoKey id) {
		dpianificFerieAmmtoRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DpianificFerieAmmtoKey> ids) {
		dpianificFerieAmmtoRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DpianificFerieAmmto> findByObjectKey(String id) {
		DpianificFerieAmmto dpianificFerieAmmto = new DpianificFerieAmmto(id);

		return findById(dpianificFerieAmmto.getDpianificFerieAmmtoKey());
	}

	@Override
	public Page<DpianificFerieAmmto> findByTheDstatiPianificFerie(DstatiPianificFerie parentEntity, Pageable pageable) {
		return dpianificFerieAmmtoRepository.findByTheDstatiPianificFerie(parentEntity, pageable);
	}

	@Override
	public Page<DpianificFerieAmmto> findByTheDesitiRichAdesionePf(DesitiRichAdesionePf parentEntity,
			Pageable pageable) {
		return dpianificFerieAmmtoRepository.findByTheDesitiRichAdesionePf(parentEntity, pageable);
	}

	/**
	 * Convert DpianificFerieAmmtoCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DpianificFerieAmmto> createSpecification(DpianificFerieAmmtoCriteria criteria) {
		Specification<DpianificFerieAmmto> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdDpianificFerieAmmto() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getSequIdDpianificFerieAmmto(), DpianificFerieAmmto_.sequIdDpianificFerieAmmto));
			}
			if (criteria.getCodiPianificazioneFerie() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getCodiPianificazioneFerie(), DpianificFerieAmmto_.codiPianificazioneFerie));
			}
			if (criteria.getDescPianificazioneFerie() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getDescPianificazioneFerie(), DpianificFerieAmmto_.descPianificazioneFerie));
			}
			if (criteria.getFkEsitoRichAdesionePf() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getFkEsitoRichAdesionePf(), DpianificFerieAmmto_.fkEsitoRichAdesionePf));
			}
			if (criteria.getFkStatoPianificFerie() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getFkStatoPianificFerie(), DpianificFerieAmmto_.fkStatoPianificFerie));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DpianificFerieAmmto_.flagEliminato));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DpianificFerieAmmto_.utenteUltimaModifica));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DpianificFerieAmmto_.dataUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataInizioRecord(), DpianificFerieAmmto_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DpianificFerieAmmto_.dataFineRecord));
			}

			// CompositeKey Filters

			if (criteria.getTheDstatiPianificFerieObjectKey() != null) {
				DstatiPianificFerie dstatiPianificFerie = new DstatiPianificFerie(
						criteria.getTheDstatiPianificFerieObjectKey());

			}
			if (criteria.getTheDesitiRichAdesionePfObjectKey() != null) {
				DesitiRichAdesionePf desitiRichAdesionePf = new DesitiRichAdesionePf(
						criteria.getTheDesitiRichAdesionePfObjectKey());

			}

		}
		return specification;
	}

	/*
	 * 
	 * @see it.micegroup.gestioneferie.pianificazioneferie.service.
	 * DpianificFerieAmmtoService#findDstatiPianificFerieBytheDesitiRichAdesionePf(
	 * it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePf,
	 * org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiPianificFerie> findDstatiPianificFerieByTheDesitiRichAdesionePf(
			DesitiRichAdesionePf desitiRichAdesionePf, Pageable pageable) {
		Page<DpianificFerieAmmto> dpianificFerieAmmtoPage = dpianificFerieAmmtoRepository
				.findByTheDesitiRichAdesionePf(desitiRichAdesionePf, pageable);
		List<DstatiPianificFerie> content = dpianificFerieAmmtoPage.getContent().stream()
				.map(DpianificFerieAmmto::getTheDstatiPianificFerie).collect(Collectors.toList());
		Page<DstatiPianificFerie> result = new PageImpl<DstatiPianificFerie>(content, pageable,
				dpianificFerieAmmtoPage.getTotalElements());
		return result;
	}

	/*
	 * 
	 * @see it.micegroup.gestioneferie.pianificazioneferie.service.
	 * DpianificFerieAmmtoService#findDesitiRichAdesionePfBytheDstatiPianificFerie(
	 * it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerie,
	 * org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DesitiRichAdesionePf> findDesitiRichAdesionePfByTheDstatiPianificFerie(
			DstatiPianificFerie dstatiPianificFerie, Pageable pageable) {
		Page<DpianificFerieAmmto> dpianificFerieAmmtoPage = dpianificFerieAmmtoRepository
				.findByTheDstatiPianificFerie(dstatiPianificFerie, pageable);
		List<DesitiRichAdesionePf> content = dpianificFerieAmmtoPage.getContent().stream()
				.map(DpianificFerieAmmto::getTheDesitiRichAdesionePf).collect(Collectors.toList());
		Page<DesitiRichAdesionePf> result = new PageImpl<DesitiRichAdesionePf>(content, pageable,
				dpianificFerieAmmtoPage.getTotalElements());
		return result;
	}

	@Override
	@Transactional()
	public DpianificFerieAmmto bulkUpdate(DpianificFerieAmmto dpianificFerieAmmto) {
		List<DrichPianifFerieAmmto> updateTheDrichPianifFerieAmmto = dpianificFerieAmmto.getTheDrichPianifFerieAmmto()
				.stream().filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DrichPianifFerieAmmto> deleteTheDrichPianifFerieAmmto = dpianificFerieAmmto.getTheDrichPianifFerieAmmto()
				.stream().filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		dpianificFerieAmmto.setTheDrichPianifFerieAmmto(updateTheDrichPianifFerieAmmto);
		List<DperiodiAmmtoPf> updateTheDperiodiAmmtoPf = dpianificFerieAmmto.getTheDperiodiAmmtoPf().stream()
				.filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DperiodiAmmtoPf> deleteTheDperiodiAmmtoPf = dpianificFerieAmmto.getTheDperiodiAmmtoPf().stream()
				.filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		dpianificFerieAmmto.setTheDperiodiAmmtoPf(updateTheDperiodiAmmtoPf);

		DpianificFerieAmmto update = this.update(dpianificFerieAmmto);

		deleteTheDrichPianifFerieAmmto
				.forEach(child -> drichPianifFerieAmmtoService.deleteById(child.getDrichPianifFerieAmmtoKey()));
		deleteTheDperiodiAmmtoPf.forEach(child -> dperiodiAmmtoPfService.deleteById(child.getDperiodiAmmtoPfKey()));
		return update;
	}
}
