package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.voila2runtime.filter.LongFilter;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePf;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePfKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePf;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianiFerie;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePfKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DrichAdesionePfCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DrichAdesionePfRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePfKey_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePfKey_;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianiFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerieKey_;

import org.springframework.data.domain.PageImpl;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePfKey_;

import java.util.stream.Collectors;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePf;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPf;
import it.micegroup.voila2runtime.entity.GenericEntity;
import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DrichAdesionePfServiceImpl extends BaseServiceImpl implements DrichAdesionePfService {

	private final DrichAdesionePfRepository drichAdesionePfRepository;
	private final FilterService<DrichAdesionePf> filterService;

	// CHILD SERVICES
	private final DesitiRichAdesionePfService desitiRichAdesionePfService;
	private final DsollecitiPfService dsollecitiPfService;

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DrichAdesionePfServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DrichAdesionePf> findAll(Pageable pageable) {
		return drichAdesionePfRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DrichAdesionePf> findAll() {
		return drichAdesionePfRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DrichAdesionePf> findByCriteria(DrichAdesionePfCriteria criteria, Pageable pageable) {
		final Specification<DrichAdesionePf> specification = createSpecification(criteria);
		return drichAdesionePfRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DrichAdesionePf> findByCriteria(DrichAdesionePfCriteria criteria) {
		final Specification<DrichAdesionePf> specification = createSpecification(criteria);
		return drichAdesionePfRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DrichAdesionePf> findById(DrichAdesionePfKey id) {
		return drichAdesionePfRepository.findByDrichAdesionePfKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DrichAdesionePfKey id) {
		return drichAdesionePfRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DrichAdesionePf insert(@Valid DrichAdesionePf entity) {
		return drichAdesionePfRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DrichAdesionePf update(@Valid DrichAdesionePf entity) {
		return drichAdesionePfRepository.save(entity);
	}

	@Override
	public void delete(DrichAdesionePf entity) {
		drichAdesionePfRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DrichAdesionePfKey id) {
		drichAdesionePfRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DrichAdesionePfKey> ids) {
		drichAdesionePfRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DrichAdesionePf> findByObjectKey(String id) {
		DrichAdesionePf drichAdesionePf = new DrichAdesionePf(id);

		return findById(drichAdesionePf.getDrichAdesionePfKey());
	}

	@Override
	public Page<DrichAdesionePf> findByTheDstatiRichAdesionePf(DstatiRichAdesionePf parentEntity, Pageable pageable) {
		return drichAdesionePfRepository.findByTheDstatiRichAdesionePf(parentEntity, pageable);
	}

	@Override
	public Page<DrichAdesionePf> findByTheDpianiFerie(DpianiFerie parentEntity, Pageable pageable) {
		return drichAdesionePfRepository.findByTheDpianiFerie(parentEntity, pageable);
	}

	/**
	 * Convert DrichAdesionePfCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DrichAdesionePf> createSpecification(DrichAdesionePfCriteria criteria) {
		Specification<DrichAdesionePf> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdRichAdesionePf() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getSequIdRichAdesionePf(), DrichAdesionePf_.sequIdRichAdesionePf));
			}
			if (criteria.getCodiRichiestaAdesione() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getCodiRichiestaAdesione(), DrichAdesionePf_.codiRichiestaAdesione));
			}
			if (criteria.getDataRichiestaAdesione() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataRichiestaAdesione(), DrichAdesionePf_.dataRichiestaAdesione));
			}
			if (criteria.getFkPianoFerie() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getFkPianoFerie(),
						DrichAdesionePf_.fkPianoFerie));
			}
			if (criteria.getFkStatoRichAdesione() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getFkStatoRichAdesione(), DrichAdesionePf_.fkStatoRichAdesione));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DrichAdesionePf_.flagEliminato));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DrichAdesionePf_.utenteUltimaModifica));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DrichAdesionePf_.dataUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService
						.generateRangeSpecification(criteria.getDataInizioRecord(), DrichAdesionePf_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DrichAdesionePf_.dataFineRecord));
			}

			// CompositeKey Filters

			if (criteria.getTheDstatiRichAdesionePfObjectKey() != null) {
				DstatiRichAdesionePf dstatiRichAdesionePf = new DstatiRichAdesionePf(
						criteria.getTheDstatiRichAdesionePfObjectKey());

			}
			if (criteria.getTheDpianiFerieObjectKey() != null) {
				DpianiFerie dpianiFerie = new DpianiFerie(criteria.getTheDpianiFerieObjectKey());

				specification = specification
						.and(filterService.generateEqualsSpecification(dpianiFerie.getSequIdPianoFerie(),
								r -> r.get(DrichAdesionePf_.theDpianiFerie).get(DpianiFerie_.sequIdPianoFerie)));
			}

		}
		return specification;
	}

	/*
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.DrichAdesionePfService
	 * #findDstatiRichAdesionePfBytheDpianiFerie(it.micegroup.gestioneferie.
	 * pianificazioneferie.entity.DpianiFerie,
	 * org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiRichAdesionePf> findDstatiRichAdesionePfByTheDpianiFerie(DpianiFerie dpianiFerie,
			Pageable pageable) {
		Page<DrichAdesionePf> drichAdesionePfPage = drichAdesionePfRepository.findByTheDpianiFerie(dpianiFerie,
				pageable);
		List<DstatiRichAdesionePf> content = drichAdesionePfPage.getContent().stream()
				.map(DrichAdesionePf::getTheDstatiRichAdesionePf).collect(Collectors.toList());
		Page<DstatiRichAdesionePf> result = new PageImpl<DstatiRichAdesionePf>(content, pageable,
				drichAdesionePfPage.getTotalElements());
		return result;
	}

	/*
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.DrichAdesionePfService
	 * #findDpianiFerieBytheDstatiRichAdesionePf(it.micegroup.gestioneferie.
	 * pianificazioneferie.entity.DstatiRichAdesionePf,
	 * org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DpianiFerie> findDpianiFerieByTheDstatiRichAdesionePf(DstatiRichAdesionePf dstatiRichAdesionePf,
			Pageable pageable) {
		Page<DrichAdesionePf> drichAdesionePfPage = drichAdesionePfRepository
				.findByTheDstatiRichAdesionePf(dstatiRichAdesionePf, pageable);
		List<DpianiFerie> content = drichAdesionePfPage.getContent().stream().map(DrichAdesionePf::getTheDpianiFerie)
				.collect(Collectors.toList());
		Page<DpianiFerie> result = new PageImpl<DpianiFerie>(content, pageable, drichAdesionePfPage.getTotalElements());
		return result;
	}

	@Override
	@Transactional()
	public DrichAdesionePf bulkUpdate(DrichAdesionePf drichAdesionePf) {
		List<DesitiRichAdesionePf> updateTheDesitiRichAdesionePf = drichAdesionePf.getTheDesitiRichAdesionePf().stream()
				.filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DesitiRichAdesionePf> deleteTheDesitiRichAdesionePf = drichAdesionePf.getTheDesitiRichAdesionePf().stream()
				.filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		drichAdesionePf.setTheDesitiRichAdesionePf(updateTheDesitiRichAdesionePf);
		List<DsollecitiPf> updateTheDsollecitiPf = drichAdesionePf.getTheDsollecitiPf().stream()
				.filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DsollecitiPf> deleteTheDsollecitiPf = drichAdesionePf.getTheDsollecitiPf().stream()
				.filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		drichAdesionePf.setTheDsollecitiPf(updateTheDsollecitiPf);

		DrichAdesionePf update = this.update(drichAdesionePf);

		deleteTheDesitiRichAdesionePf
				.forEach(child -> desitiRichAdesionePfService.deleteById(child.getDesitiRichAdesionePfKey()));
		deleteTheDsollecitiPf.forEach(child -> dsollecitiPfService.deleteById(child.getDsollecitiPfKey()));
		return update;
	}
}
