package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPf;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPfKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePf;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePfKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPf_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DsollecitiPfCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DsollecitiPfRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPfKey_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichAdesionePfKey_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichAdesionePfKey_;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianiFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerieKey_;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DsollecitiPfKey_;

import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DsollecitiPfServiceImpl extends BaseServiceImpl implements DsollecitiPfService {

	private final DsollecitiPfRepository dsollecitiPfRepository;
	private final FilterService<DsollecitiPf> filterService;

	// CHILD SERVICES

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DsollecitiPfServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DsollecitiPf> findAll(Pageable pageable) {
		return dsollecitiPfRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DsollecitiPf> findAll() {
		return dsollecitiPfRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DsollecitiPf> findByCriteria(DsollecitiPfCriteria criteria, Pageable pageable) {
		final Specification<DsollecitiPf> specification = createSpecification(criteria);
		return dsollecitiPfRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DsollecitiPf> findByCriteria(DsollecitiPfCriteria criteria) {
		final Specification<DsollecitiPf> specification = createSpecification(criteria);
		return dsollecitiPfRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DsollecitiPf> findById(DsollecitiPfKey id) {
		return dsollecitiPfRepository.findByDsollecitiPfKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DsollecitiPfKey id) {
		return dsollecitiPfRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DsollecitiPf insert(@Valid DsollecitiPf entity) {
		return dsollecitiPfRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DsollecitiPf update(@Valid DsollecitiPf entity) {
		return dsollecitiPfRepository.save(entity);
	}

	@Override
	public void delete(DsollecitiPf entity) {
		dsollecitiPfRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DsollecitiPfKey id) {
		dsollecitiPfRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DsollecitiPfKey> ids) {
		dsollecitiPfRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DsollecitiPf> findByObjectKey(String id) {
		DsollecitiPf dsollecitiPf = new DsollecitiPf(id);

		return findById(dsollecitiPf.getDsollecitiPfKey());
	}

	@Override
	public Page<DsollecitiPf> findByTheDrichAdesionePf(DrichAdesionePf parentEntity, Pageable pageable) {
		return dsollecitiPfRepository.findByTheDrichAdesionePf(parentEntity, pageable);
	}

	/**
	 * Convert DsollecitiPfCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DsollecitiPf> createSpecification(DsollecitiPfCriteria criteria) {
		Specification<DsollecitiPf> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdSollecitoPf() != null) {
				specification = specification.and(filterService
						.generateRangeSpecification(criteria.getSequIdSollecitoPf(), DsollecitiPf_.sequIdSollecitoPf));
			}
			if (criteria.getCodiSollecito() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getCodiSollecito(),
						DsollecitiPf_.codiSollecito));
			}
			if (criteria.getDataSollecito() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataSollecito(),
						DsollecitiPf_.dataSollecito));
			}
			if (criteria.getFkAmministrato() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getFkAmministrato(),
						DsollecitiPf_.fkAmministrato));
			}
			if (criteria.getFkRichiestaAdesione() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getFkRichiestaAdesione(), DsollecitiPf_.fkRichiestaAdesione));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DsollecitiPf_.flagEliminato));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DsollecitiPf_.utenteUltimaModifica));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DsollecitiPf_.dataUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService
						.generateRangeSpecification(criteria.getDataInizioRecord(), DsollecitiPf_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DsollecitiPf_.dataFineRecord));
			}

			// CompositeKey Filters

			if (criteria.getTheDrichAdesionePfObjectKey() != null) {
				DrichAdesionePf drichAdesionePf = new DrichAdesionePf(criteria.getTheDrichAdesionePfObjectKey());

			}

		}
		return specification;
	}

	@Override
	@Transactional()
	public DsollecitiPf bulkUpdate(DsollecitiPf dsollecitiPf) {

		DsollecitiPf update = this.update(dsollecitiPf);

		return update;
	}
}
