package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePf;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePfKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DstatiAdesionePfCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DstatiAdesionePfRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePf_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePfKey_;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiAdesionePfKey_;

import java.util.stream.Collectors;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DesitiRichAdesionePf;
import it.micegroup.voila2runtime.entity.GenericEntity;
import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DstatiAdesionePfServiceImpl extends BaseServiceImpl implements DstatiAdesionePfService {

	private final DstatiAdesionePfRepository dstatiAdesionePfRepository;
	private final FilterService<DstatiAdesionePf> filterService;

	// CHILD SERVICES
	private final DesitiRichAdesionePfService desitiRichAdesionePfService;

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DstatiAdesionePfServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiAdesionePf> findAll(Pageable pageable) {
		return dstatiAdesionePfRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiAdesionePf> findAll() {
		return dstatiAdesionePfRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiAdesionePf> findByCriteria(DstatiAdesionePfCriteria criteria, Pageable pageable) {
		final Specification<DstatiAdesionePf> specification = createSpecification(criteria);
		return dstatiAdesionePfRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiAdesionePf> findByCriteria(DstatiAdesionePfCriteria criteria) {
		final Specification<DstatiAdesionePf> specification = createSpecification(criteria);
		return dstatiAdesionePfRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiAdesionePf> findById(DstatiAdesionePfKey id) {
		return dstatiAdesionePfRepository.findByDstatiAdesionePfKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DstatiAdesionePfKey id) {
		return dstatiAdesionePfRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DstatiAdesionePf insert(@Valid DstatiAdesionePf entity) {
		return dstatiAdesionePfRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DstatiAdesionePf update(@Valid DstatiAdesionePf entity) {
		return dstatiAdesionePfRepository.save(entity);
	}

	@Override
	public void delete(DstatiAdesionePf entity) {
		dstatiAdesionePfRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DstatiAdesionePfKey id) {
		dstatiAdesionePfRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DstatiAdesionePfKey> ids) {
		dstatiAdesionePfRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiAdesionePf> findByObjectKey(String id) {
		DstatiAdesionePf dstatiAdesionePf = new DstatiAdesionePf(id);

		return findById(dstatiAdesionePf.getDstatiAdesionePfKey());
	}

	/**
	 * Convert DstatiAdesionePfCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DstatiAdesionePf> createSpecification(DstatiAdesionePfCriteria criteria) {
		Specification<DstatiAdesionePf> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdStatoAdesionePf() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getSequIdStatoAdesionePf(), DstatiAdesionePf_.sequIdStatoAdesionePf));
			}
			if (criteria.getCodiStatoAdesione() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getCodiStatoAdesione(), DstatiAdesionePf_.codiStatoAdesione));
			}
			if (criteria.getDescStatoAdesione() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getDescStatoAdesione(), DstatiAdesionePf_.descStatoAdesione));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DstatiAdesionePf_.flagEliminato));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DstatiAdesionePf_.utenteUltimaModifica));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DstatiAdesionePf_.dataUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataInizioRecord(), DstatiAdesionePf_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DstatiAdesionePf_.dataFineRecord));
			}

			// CompositeKey Filters

		}
		return specification;
	}

	@Override
	@Transactional()
	public DstatiAdesionePf bulkUpdate(DstatiAdesionePf dstatiAdesionePf) {
		List<DesitiRichAdesionePf> updateTheDesitiRichAdesionePf = dstatiAdesionePf.getTheDesitiRichAdesionePf()
				.stream().filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DesitiRichAdesionePf> deleteTheDesitiRichAdesionePf = dstatiAdesionePf.getTheDesitiRichAdesionePf()
				.stream().filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		dstatiAdesionePf.setTheDesitiRichAdesionePf(updateTheDesitiRichAdesionePf);

		DstatiAdesionePf update = this.update(dstatiAdesionePf);

		deleteTheDesitiRichAdesionePf
				.forEach(child -> desitiRichAdesionePfService.deleteById(child.getDesitiRichAdesionePfKey()));
		return update;
	}
}
