package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerie;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerieKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DstatiPianificFerieCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DstatiPianificFerieRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerieKey_;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianificFerieKey_;

import java.util.stream.Collectors;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianificFerieAmmto;
import it.micegroup.voila2runtime.entity.GenericEntity;
import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DstatiPianificFerieServiceImpl extends BaseServiceImpl implements DstatiPianificFerieService {

	private final DstatiPianificFerieRepository dstatiPianificFerieRepository;
	private final FilterService<DstatiPianificFerie> filterService;

	// CHILD SERVICES
	private final DpianificFerieAmmtoService dpianificFerieAmmtoService;

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DstatiPianificFerieServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiPianificFerie> findAll(Pageable pageable) {
		return dstatiPianificFerieRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiPianificFerie> findAll() {
		return dstatiPianificFerieRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiPianificFerie> findByCriteria(DstatiPianificFerieCriteria criteria, Pageable pageable) {
		final Specification<DstatiPianificFerie> specification = createSpecification(criteria);
		return dstatiPianificFerieRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiPianificFerie> findByCriteria(DstatiPianificFerieCriteria criteria) {
		final Specification<DstatiPianificFerie> specification = createSpecification(criteria);
		return dstatiPianificFerieRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiPianificFerie> findById(DstatiPianificFerieKey id) {
		return dstatiPianificFerieRepository.findByDstatiPianificFerieKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DstatiPianificFerieKey id) {
		return dstatiPianificFerieRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DstatiPianificFerie insert(@Valid DstatiPianificFerie entity) {
		return dstatiPianificFerieRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DstatiPianificFerie update(@Valid DstatiPianificFerie entity) {
		return dstatiPianificFerieRepository.save(entity);
	}

	@Override
	public void delete(DstatiPianificFerie entity) {
		dstatiPianificFerieRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DstatiPianificFerieKey id) {
		dstatiPianificFerieRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DstatiPianificFerieKey> ids) {
		dstatiPianificFerieRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiPianificFerie> findByObjectKey(String id) {
		DstatiPianificFerie dstatiPianificFerie = new DstatiPianificFerie(id);

		return findById(dstatiPianificFerie.getDstatiPianificFerieKey());
	}

	/**
	 * Convert DstatiPianificFerieCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DstatiPianificFerie> createSpecification(DstatiPianificFerieCriteria criteria) {
		Specification<DstatiPianificFerie> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdStatoPianificFerie() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getSequIdStatoPianificFerie(), DstatiPianificFerie_.sequIdStatoPianificFerie));
			}
			if (criteria.getCodiStatoPianificazione() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getCodiStatoPianificazione(), DstatiPianificFerie_.codiStatoPianificazione));
			}
			if (criteria.getDescStatoPianificazione() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getDescStatoPianificazione(), DstatiPianificFerie_.descStatoPianificazione));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DstatiPianificFerie_.flagEliminato));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DstatiPianificFerie_.utenteUltimaModifica));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DstatiPianificFerie_.dataUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataInizioRecord(), DstatiPianificFerie_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DstatiPianificFerie_.dataFineRecord));
			}

			// CompositeKey Filters

		}
		return specification;
	}

	@Override
	@Transactional()
	public DstatiPianificFerie bulkUpdate(DstatiPianificFerie dstatiPianificFerie) {
		List<DpianificFerieAmmto> updateTheDpianificFerieAmmto = dstatiPianificFerie.getTheDpianificFerieAmmto()
				.stream().filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DpianificFerieAmmto> deleteTheDpianificFerieAmmto = dstatiPianificFerie.getTheDpianificFerieAmmto()
				.stream().filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		dstatiPianificFerie.setTheDpianificFerieAmmto(updateTheDpianificFerieAmmto);

		DstatiPianificFerie update = this.update(dstatiPianificFerie);

		deleteTheDpianificFerieAmmto
				.forEach(child -> dpianificFerieAmmtoService.deleteById(child.getDpianificFerieAmmtoKey()));
		return update;
	}
}
