package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerie;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerieKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DstatiPianoFerieCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DstatiPianoFerieRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerie_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerieKey_;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiPianoFerieKey_;

import java.util.stream.Collectors;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DpianiFerie;
import it.micegroup.voila2runtime.entity.GenericEntity;
import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DstatiPianoFerieServiceImpl extends BaseServiceImpl implements DstatiPianoFerieService {

	private final DstatiPianoFerieRepository dstatiPianoFerieRepository;
	private final FilterService<DstatiPianoFerie> filterService;

	// CHILD SERVICES
	private final DpianiFerieService dpianiFerieService;

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DstatiPianoFerieServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiPianoFerie> findAll(Pageable pageable) {
		return dstatiPianoFerieRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiPianoFerie> findAll() {
		return dstatiPianoFerieRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiPianoFerie> findByCriteria(DstatiPianoFerieCriteria criteria, Pageable pageable) {
		final Specification<DstatiPianoFerie> specification = createSpecification(criteria);
		return dstatiPianoFerieRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiPianoFerie> findByCriteria(DstatiPianoFerieCriteria criteria) {
		final Specification<DstatiPianoFerie> specification = createSpecification(criteria);
		return dstatiPianoFerieRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiPianoFerie> findById(DstatiPianoFerieKey id) {
		return dstatiPianoFerieRepository.findByDstatiPianoFerieKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DstatiPianoFerieKey id) {
		return dstatiPianoFerieRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DstatiPianoFerie insert(@Valid DstatiPianoFerie entity) {
		return dstatiPianoFerieRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DstatiPianoFerie update(@Valid DstatiPianoFerie entity) {
		return dstatiPianoFerieRepository.save(entity);
	}

	@Override
	public void delete(DstatiPianoFerie entity) {
		dstatiPianoFerieRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DstatiPianoFerieKey id) {
		dstatiPianoFerieRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DstatiPianoFerieKey> ids) {
		dstatiPianoFerieRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiPianoFerie> findByObjectKey(String id) {
		DstatiPianoFerie dstatiPianoFerie = new DstatiPianoFerie(id);

		return findById(dstatiPianoFerie.getDstatiPianoFerieKey());
	}

	/**
	 * Convert DstatiPianoFerieCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DstatiPianoFerie> createSpecification(DstatiPianoFerieCriteria criteria) {
		Specification<DstatiPianoFerie> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdStatoPianoFerie() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getSequIdStatoPianoFerie(), DstatiPianoFerie_.sequIdStatoPianoFerie));
			}
			if (criteria.getCodiStatoPianoFerie() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getCodiStatoPianoFerie(), DstatiPianoFerie_.codiStatoPianoFerie));
			}
			if (criteria.getDescStatoPianoFerie() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getDescStatoPianoFerie(), DstatiPianoFerie_.descStatoPianoFerie));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DstatiPianoFerie_.utenteUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataInizioRecord(), DstatiPianoFerie_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DstatiPianoFerie_.dataFineRecord));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DstatiPianoFerie_.dataUltimaModifica));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DstatiPianoFerie_.flagEliminato));
			}

			// CompositeKey Filters

		}
		return specification;
	}

	@Override
	@Transactional()
	public DstatiPianoFerie bulkUpdate(DstatiPianoFerie dstatiPianoFerie) {
		List<DpianiFerie> updateTheDpianiFerie = dstatiPianoFerie.getTheDpianiFerie().stream()
				.filter(child -> !child.isDeletedEntityState()).collect(Collectors.toList());
		List<DpianiFerie> deleteTheDpianiFerie = dstatiPianoFerie.getTheDpianiFerie().stream()
				.filter(GenericEntity::isDeletedEntityState).collect(Collectors.toList());

		dstatiPianoFerie.setTheDpianiFerie(updateTheDpianiFerie);

		DstatiPianoFerie update = this.update(dstatiPianoFerie);

		deleteTheDpianiFerie.forEach(child -> dpianiFerieService.deleteById(child.getSequIdPianoFerie()));
		return update;
	}
}
