package it.micegroup.gestioneferie.pianificazioneferie.service;

import java.util.Map;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Optional;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

import org.springframework.context.i18n.LocaleContextHolder;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichiestaPfAmmto;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichiestaPfAmmtoKey;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichiestaPfAmmto_;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DstatiRichiestaPfAmmtoCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.repository.DstatiRichiestaPfAmmtoRepository;
import it.micegroup.voila2runtime.specification.FilterService;

import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichiestaPfAmmto_;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichiestaPfAmmtoKey_;
import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.libs.service.BaseServiceImpl;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DstatiRichiestaPfAmmtoKey_;

import java.util.stream.Collectors;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DrichPianifFerieAmmto;
import it.micegroup.voila2runtime.entity.GenericEntity;
import it.micegroup.voila2runtime.exception.BusinessException;
import net.sf.jasperreports.engine.JRException;

@RequiredArgsConstructor
@Service
@Transactional
public class DstatiRichiestaPfAmmtoServiceImpl extends BaseServiceImpl implements DstatiRichiestaPfAmmtoService {

	private final DstatiRichiestaPfAmmtoRepository dstatiRichiestaPfAmmtoRepository;
	private final FilterService<DstatiRichiestaPfAmmto> filterService;

	// CHILD SERVICES
	private final DrichPianifFerieAmmtoService drichPianifFerieAmmtoService;

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DstatiRichiestaPfAmmtoServiceImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll(org. springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiRichiestaPfAmmto> findAll(Pageable pageable) {
		return dstatiRichiestaPfAmmtoRepository.findAll(pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findAll()
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiRichiestaPfAmmto> findAll() {
		return dstatiRichiestaPfAmmtoRepository.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object, org.springframework.data.domain.Pageable)
	 */
	@Override
	@Transactional(readOnly = true)
	public Page<DstatiRichiestaPfAmmto> findByCriteria(DstatiRichiestaPfAmmtoCriteria criteria, Pageable pageable) {
		final Specification<DstatiRichiestaPfAmmto> specification = createSpecification(criteria);
		return dstatiRichiestaPfAmmtoRepository.findAll(specification, pageable);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByCriteria(java. lang.Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public List<DstatiRichiestaPfAmmto> findByCriteria(DstatiRichiestaPfAmmtoCriteria criteria) {
		final Specification<DstatiRichiestaPfAmmto> specification = createSpecification(criteria);
		return dstatiRichiestaPfAmmtoRepository.findAll(specification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findById(java.lang. Object)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiRichiestaPfAmmto> findById(DstatiRichiestaPfAmmtoKey id) {
		return dstatiRichiestaPfAmmtoRepository.findByDstatiRichiestaPfAmmtoKey(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * exists(java.lang. Object)
	 */
	@Override
	public boolean exists(DstatiRichiestaPfAmmtoKey id) {
		return dstatiRichiestaPfAmmtoRepository.existsById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * insert(java. lang.Object)
	 */
	@Override
	public DstatiRichiestaPfAmmto insert(@Valid DstatiRichiestaPfAmmto entity) {
		return dstatiRichiestaPfAmmtoRepository.save(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * update(java. lang.Object)
	 */
	@Override
	public DstatiRichiestaPfAmmto update(@Valid DstatiRichiestaPfAmmto entity) {
		return dstatiRichiestaPfAmmtoRepository.save(entity);
	}

	@Override
	public void delete(DstatiRichiestaPfAmmto entity) {
		dstatiRichiestaPfAmmtoRepository.delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteById(java.lang. Object)
	 */
	@Override
	public void deleteById(DstatiRichiestaPfAmmtoKey id) {
		dstatiRichiestaPfAmmtoRepository.deleteById(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * deleteByIdIn(java.lang. Object)
	 */
	@Override
	public void deleteByIdIn(Collection<DstatiRichiestaPfAmmtoKey> ids) {
		dstatiRichiestaPfAmmtoRepository.deleteByIdIn(ids);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * it.micegroup.gestioneferie.pianificazioneferie.service.GenericEntityService#
	 * findByObjectKey(java.lang.String)
	 */
	@Override
	@Transactional(readOnly = true)
	public Optional<DstatiRichiestaPfAmmto> findByObjectKey(String id) {
		DstatiRichiestaPfAmmto dstatiRichiestaPfAmmto = new DstatiRichiestaPfAmmto(id);

		return findById(dstatiRichiestaPfAmmto.getDstatiRichiestaPfAmmtoKey());
	}

	/**
	 * Convert DstatiRichiestaPfAmmtoCriteria to a {@link Specification}
	 * 
	 * @param criteria
	 * @return
	 */
	private Specification<DstatiRichiestaPfAmmto> createSpecification(DstatiRichiestaPfAmmtoCriteria criteria) {
		Specification<DstatiRichiestaPfAmmto> specification = Specification.where(null);

		if (criteria != null) {
			// Attribute filters
			if (criteria.getSequIdStatoRichPfAmmto() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getSequIdStatoRichPfAmmto(), DstatiRichiestaPfAmmto_.sequIdStatoRichPfAmmto));
			}
			if (criteria.getCodiStatoRichiesta() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getCodiStatoRichiesta(), DstatiRichiestaPfAmmto_.codiStatoRichiesta));
			}
			if (criteria.getDescStatoRichiesta() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getDescStatoRichiesta(), DstatiRichiestaPfAmmto_.descStatoRichiesta));
			}
			if (criteria.getFlagEliminato() != null) {
				specification = specification.and(filterService.generateStringSpecification(criteria.getFlagEliminato(),
						DstatiRichiestaPfAmmto_.flagEliminato));
			}
			if (criteria.getUtenteUltimaModifica() != null) {
				specification = specification.and(filterService.generateStringSpecification(
						criteria.getUtenteUltimaModifica(), DstatiRichiestaPfAmmto_.utenteUltimaModifica));
			}
			if (criteria.getDataUltimaModifica() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataUltimaModifica(), DstatiRichiestaPfAmmto_.dataUltimaModifica));
			}
			if (criteria.getDataInizioRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(
						criteria.getDataInizioRecord(), DstatiRichiestaPfAmmto_.dataInizioRecord));
			}
			if (criteria.getDataFineRecord() != null) {
				specification = specification.and(filterService.generateRangeSpecification(criteria.getDataFineRecord(),
						DstatiRichiestaPfAmmto_.dataFineRecord));
			}

			// CompositeKey Filters

		}
		return specification;
	}

	@Override
	@Transactional()
	public DstatiRichiestaPfAmmto bulkUpdate(DstatiRichiestaPfAmmto dstatiRichiestaPfAmmto) {
		List<DrichPianifFerieAmmto> updateTheDrichPianifFerieAmmto = dstatiRichiestaPfAmmto
				.getTheDrichPianifFerieAmmto().stream().filter(child -> !child.isDeletedEntityState())
				.collect(Collectors.toList());
		List<DrichPianifFerieAmmto> deleteTheDrichPianifFerieAmmto = dstatiRichiestaPfAmmto
				.getTheDrichPianifFerieAmmto().stream().filter(GenericEntity::isDeletedEntityState)
				.collect(Collectors.toList());

		dstatiRichiestaPfAmmto.setTheDrichPianifFerieAmmto(updateTheDrichPianifFerieAmmto);

		DstatiRichiestaPfAmmto update = this.update(dstatiRichiestaPfAmmto);

		deleteTheDrichPianifFerieAmmto
				.forEach(child -> drichPianifFerieAmmtoService.deleteById(child.getDrichPianifFerieAmmtoKey()));
		return update;
	}
}
