package it.micegroup.gestioneferie.pianificazioneferie.controller;

import lombok.RequiredArgsConstructor;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DmotivazioniAlertPf;
import it.micegroup.gestioneferie.pianificazioneferie.assembler.DmotivazioniAlertPfModelAssembler;
import it.micegroup.gestioneferie.pianificazioneferie.criteria.DmotivazioniAlertPfCriteria;
import it.micegroup.gestioneferie.pianificazioneferie.service.DmotivazioniAlertPfService;
import it.micegroup.gestioneferie.pianificazioneferie.service.DammtiPianiFerieService;
import it.micegroup.gestioneferie.pianificazioneferie.entity.DammtiPianiFerie;

import it.micegroup.voila2runtime.exception.*;
import it.micegroup.gestioneferie.libs.controller.BaseController;
import it.micegroup.gestioneferie.pianificazioneferie.dto.DmotivazioniAlertPfDto;
import it.micegroup.gestioneferie.pianificazioneferie.dto.DmotivazioniAlertPfWriteDto;
import it.micegroup.gestioneferie.libs.response.BaseResponse;
import it.micegroup.gestioneferie.libs.request.BaseRequest;
import javax.annotation.PostConstruct;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PagedResourcesAssembler;
import org.springframework.hateoas.PagedModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import javax.validation.Valid;
import java.net.URI;
import java.util.Locale;
import java.util.Optional;

import java.util.Collection;
import org.modelmapper.PropertyMap;
import java.util.stream.Collectors;

@RequiredArgsConstructor
@RestController
@RequestMapping(value = "/dmotivazioni-alert-pf", produces = MediaType.APPLICATION_JSON_VALUE)
public class DmotivazioniAlertPfController extends BaseController<BaseResponse<DmotivazioniAlertPfDto>> {

	/// ENTITY SERVICE
	private final DmotivazioniAlertPfService dmotivazioniAlertPfService;
	private final DmotivazioniAlertPfModelAssembler dmotivazioniAlertPfModelAssembler;
	private final PagedResourcesAssembler<DmotivazioniAlertPf> pagedResourcesAssembler;
	private final ModelMapper modelMapper;
	// CHILD SERVICES
	private final DammtiPianiFerieService dammtiPianiFerieService;

	// CONSTRUCTOR
	@PostConstruct
	public void postConstruct() {
		modelMapper.addMappings(new PropertyMap<DmotivazioniAlertPfWriteDto, DmotivazioniAlertPf>() {
			@Override
			protected void configure() {
				skip(destination.getTheDammtiPianiFerie());
			}
		});
	}

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(DmotivazioniAlertPfController.class);

	// API
	@GetMapping
	@PreAuthorize("hasRole(@permissionHolder.DMOTIVAZIONI_ALERT_PF_FIND_PAGINATED.toString())")
	public ResponseEntity<BaseResponse<PagedModel<DmotivazioniAlertPfDto>>> findPaginated(
			DmotivazioniAlertPfCriteria criteria, Pageable pageable) {
		Page<DmotivazioniAlertPf> page = dmotivazioniAlertPfService.findByCriteria(criteria, pageable);
		return toResponseEntityPaged(page, null);
	}

	/**
	 * {@code GET  /dmotivazioni-alert-pf/:objectKey} : get the
	 * dmotivazioni-alert-pf with given objectKey.
	 *
	 * @param objectKey the objectKey of the dmotivazioni-alert-pf to retrieve.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the dmotivazioni-alert-pf, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/{objectKey:.+}")
	@PreAuthorize("hasRole(@permissionHolder.DMOTIVAZIONI_ALERT_PF_FIND_BY_ID.toString())")
	public ResponseEntity<BaseResponse<DmotivazioniAlertPfDto>> getByObjectKey(@PathVariable String objectKey) {
		return toResponseEntity(dmotivazioniAlertPfService.findByObjectKey(objectKey), null, HttpStatus.OK);
	}

	/**
	 * {@code PUT  /dmotivazioni-alert-pf} : Updates an existing
	 * DmotivazioniAlertPf.
	 *
	 * @param DmotivazioniAlertPf the DmotivazioniAlertPf to update.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated DmotivazioniAlertPf, or with status
	 *         {@code 400 (Bad Request)} if the DmotivazioniAlertPf is not valid, or
	 *         with status {@code 500 (Internal Server Error)} if the
	 *         DmotivazioniAlertPf couldn't be updated.
	 */
	@PutMapping
	@PreAuthorize("hasRole(@permissionHolder.DMOTIVAZIONI_ALERT_PF_UPDATE.toString())")
	public ResponseEntity<?> update(@Valid @RequestBody BaseRequest<DmotivazioniAlertPfWriteDto> request) {
		return toResponseEntity(
				Optional.of(dmotivazioniAlertPfService
						.bulkUpdate(modelMapper.map(request.getData(), DmotivazioniAlertPf.class))),
				null, HttpStatus.OK);
	}

	/**
	 * {@code POST  /dmotivazioni-alert-pf} : Create a new DmotivazioniAlertPf.
	 *
	 * @param DmotivazioniAlertPf the DmotivazioniAlertPf to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new DmotivazioniAlertPf, or with status
	 *         {@code 400 (Bad Request)} if the DmotivazioniAlertPf has already an
	 *         ID.
	 * @throws URISyntaxException if the Location URI syntax is incorrect.
	 */
	@PostMapping
	@PreAuthorize("hasRole(@permissionHolder.DMOTIVAZIONI_ALERT_PF_CREATE.toString())")
	public ResponseEntity<?> save(@Valid @RequestBody BaseRequest<DmotivazioniAlertPfWriteDto> request) {
		ResponseEntity<?> response = null;
		DmotivazioniAlertPf dmotivazioniAlertPfToInsert = childMap(request.getData());
		try {
			DmotivazioniAlertPf newDmotivazioniAlertPf = dmotivazioniAlertPfService
					.insert(modelMapper.map(dmotivazioniAlertPfToInsert, DmotivazioniAlertPf.class));
			URI location = ServletUriComponentsBuilder.fromCurrentRequest().path("/{id}")
					.buildAndExpand(newDmotivazioniAlertPf.getDmotivazioniAlertPfKey()).toUri();
			HttpHeaders headers = new HttpHeaders();
			headers.setLocation(location);
			response = toResponseEntity(Optional.of(newDmotivazioniAlertPf), headers, HttpStatus.CREATED);
		} catch (Exception exception) {
			LOGGER.error("Exception:", exception);
		}
		return response;
	}

	private DmotivazioniAlertPf childMap(DmotivazioniAlertPfWriteDto data) {
		DmotivazioniAlertPf dmotivazioniAlertPfToInsert = modelMapper.map(data, DmotivazioniAlertPf.class);
		Collection<DammtiPianiFerie> theDammtiPianiFeries = data.getTheDammtiPianiFerie().stream()
				.map(element -> modelMapper.map(element, DammtiPianiFerie.class))
				.peek(element -> element.setTheDmotivazioniAlertPf(dmotivazioniAlertPfToInsert))
				.collect(Collectors.toList());
		dmotivazioniAlertPfToInsert.setTheDammtiPianiFerie(theDammtiPianiFeries);
		return dmotivazioniAlertPfToInsert;
	}

	/**
	 * {@code DELETE  /dmotivazioni-alert-pf/:objectKey} : delete the
	 * dmotivazioni-alert-pf with given objectKey.
	 *
	 * @param objectKey the objectKey of the DmotivazioniAlertPf to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/{objectKey:.+}")
	@PreAuthorize("hasRole(@permissionHolder.DMOTIVAZIONI_ALERT_PF_DELETE.toString())")
	public ResponseEntity<?> delete(@PathVariable String objectKey) {
		dmotivazioniAlertPfService
				.deleteById(dmotivazioniAlertPfService.findByObjectKey(objectKey).get().getDmotivazioniAlertPfKey());
		return new ResponseEntity<>(new BaseResponse<>(objectKey), HttpStatus.OK);
	}

	/**
	 * GET /id/dammti-piani-ferie: Search all DammtiPianiFerie (childs) for the
	 * given DmotivazioniAlertPf
	 * 
	 * @param objectKey ObjectKey of DmotivazioniAlertPf
	 * @param pageable
	 * @return Page of DammtiPianiFerie for the given DmotivazioniAlertPf
	 */
	@GetMapping("/{objectKey:.+}/dammti-piani-ferie")
	@PreAuthorize("hasRole(@permissionHolder.DMOTIVAZIONI_ALERT_PF_FIND_BY_THE_DAMMTI_PIANI_FERIE_OBJECT_KEY.toString())")
	public ResponseEntity<Page<DammtiPianiFerie>> getTheDammtiPianiFerieByObjectKey(@PathVariable String objectKey,
			Pageable pageable) {
		DmotivazioniAlertPf dmotivazioniAlertPf = new DmotivazioniAlertPf();
		dmotivazioniAlertPf.setObjectKey(objectKey);
		return ResponseEntity.ok()
				.body(dammtiPianiFerieService.findByTheDmotivazioniAlertPf(dmotivazioniAlertPf, pageable));
	}

	private ResponseEntity<BaseResponse<DmotivazioniAlertPfDto>> toResponseEntity(
			Optional<DmotivazioniAlertPf> maybeResponse, HttpHeaders header, HttpStatus status) {
		return maybeResponse
				.map(response -> new ResponseEntity<>(
						new BaseResponse<>(dmotivazioniAlertPfModelAssembler.toModel(response)), header, status))
				.orElseThrow(() -> new NotFoundException(DmotivazioniAlertPf.class, new Locale("it")));
	}

	private ResponseEntity<BaseResponse<PagedModel<DmotivazioniAlertPfDto>>> toResponseEntityPaged(
			Page<DmotivazioniAlertPf> list, HttpHeaders header) {
		BaseResponse<PagedModel<DmotivazioniAlertPfDto>> collModel = new BaseResponse<>(
				pagedResourcesAssembler.toModel(list, dmotivazioniAlertPfModelAssembler));
		return ResponseEntity.ok().headers(header).body(collModel);
	}

	@SuppressWarnings("unused")
	private DmotivazioniAlertPf dtoToEntity(DmotivazioniAlertPfDto dto) {
		return new ModelMapper().map(dto, DmotivazioniAlertPf.class);
	}
}
