/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.examples.filesystem.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.internal.ui.actions.TeamAction;
import org.eclipse.ui.internal.LegacyResourceSupport;

import it.mice.voila2.sync.plugin.Activator;

/**
 * An abstract class that acts as a super class for FileSystemProvider actions.
 * It provides some general methods applicable to multiple actions.
 */
public abstract class FileSystemAction extends TeamAction {

	/**
	 * @see org.eclipse.team.internal.ui.actions.TeamAction#isEnabled()
	 */
	public boolean isEnabled() {
		return getSelectedMappings().length > 0;
	}

	/**
	 * Split the resources into sets associated with their project/provider
	 */
	protected Map getRepositoryProviderMapping() {
		HashMap result = new HashMap();
		IResource[] resources = getSelectedResources();
		for (int i = 0; i < resources.length; i++) {
			RepositoryProvider provider = RepositoryProvider.getProvider(resources[i].getProject());
			List list = (List) result.get(provider);
			if (list == null) {
				list = new ArrayList();
				result.put(provider, list);
			}
			list.add(resources[i]);
		}
		return result;
	}
	
	/**
	 * Return the selected resource mappings that are associated with the
	 * file system provider.
	 * @return the selected resource mappings that are associated with the
	 * file system provider.
	 */
	protected ResourceMapping[] getSelectedMappings() {
		return getSelectedResourceMappings(Activator.PROVIDER_ID);
	}

    protected ResourceMapping[] getSelectedResourceMappings(String providerId) {
        Object[] elements = getSelection().toArray();
        ArrayList providerMappings = new ArrayList();
        for (int i = 0; i < elements.length; i++) {
            Object object = elements[i];
            Object adapted = getResourceMapping(object);
            if (adapted instanceof ResourceMapping) {
                ResourceMapping mapping = (ResourceMapping) adapted;
                //TODO check if the selection is part of the sync
                //if (providerId == null || isMappedToProvider(mapping, providerId)) {
                    providerMappings.add(mapping);
                //}
            }
        }
        return (ResourceMapping[]) providerMappings.toArray(new ResourceMapping[providerMappings.size()]);
    }

    private Object getResourceMapping(Object object) {
        if (object instanceof ResourceMapping)
            return (ResourceMapping)object;
        return LegacyResourceSupport.getAdaptedContributorResourceMapping(object);
    }

}
